'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createMoveItemHandler = createMoveItemHandler;

var _utilities = require('./utilities');

var _tractorErrorHandler = require('tractor-error-handler');

// Dependencies:
function createMoveItemHandler(fileStructure) {
    return function moveItem(request, response) {
        var _request$body = request.body,
            copy = _request$body.copy,
            newUrl = _request$body.newUrl;

        var itemUrl = request.params[0];
        var itemPath = (0, _utilities.getItemPath)(fileStructure, itemUrl);

        var file = fileStructure.allFilesByPath[itemPath];
        var directory = fileStructure.allDirectoriesByPath[itemPath];

        if (!file && !directory) {
            return (0, _utilities.respondItemNotFound)(itemPath, response);
        }

        var toMove = file || directory;

        copy = copy || false;
        var newPath = void 0;
        if (copy) {
            newPath = (0, _utilities.getCopyPath)(toMove);
        } else {
            newPath = (0, _utilities.getItemPath)(fileStructure, newUrl);
        }

        return toMove.move({ newPath: newPath }, { isCopy: copy }).then(function () {
            return (0, _utilities.respondOkay)(response);
        }).catch(_tractorErrorHandler.TractorError.isTractorError, function (error) {
            return (0, _tractorErrorHandler.handleError)(response, error);
        }).catch(function () {
            return (0, _tractorErrorHandler.handleError)(response, new _tractorErrorHandler.TractorError('Could not move "' + itemPath + '"'));
        });
    };
}

// Errors: