'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createSaveItemHandler = createSaveItemHandler;

var _Directory = require('../structure/Directory');

var _utilities = require('./utilities');

var _tractorErrorHandler = require('tractor-error-handler');

function createSaveItemHandler(fileStructure) {
    return function saveItem(request, response) {
        var _request$body = request.body,
            data = _request$body.data,
            overwrite = _request$body.overwrite;

        var itemUrl = request.params[0];
        var itemPath = (0, _utilities.getItemPath)(fileStructure, itemUrl);

        var file = fileStructure.allFilesByPath[itemPath];
        var directory = fileStructure.allDirectoriesByPath[itemPath];
        var toSave = file || directory;
        var isDirectory = directory || data == null;

        if (toSave && !overwrite) {
            itemPath = (0, _utilities.getCopyPath)(toSave);
            toSave = null;
        }

        if (!toSave) {
            if (isDirectory) {
                toSave = new _Directory.Directory(itemPath, fileStructure);
            } else {
                var fileConstructor = fileStructure.getFileConstructor(itemPath);
                toSave = new fileConstructor(itemPath, fileStructure);
            }
        }
        return toSave.save(data).then(function () {
            return (0, _utilities.respondOkay)(response);
        }).catch(_tractorErrorHandler.TractorError.isTractorError, function (error) {
            return (0, _tractorErrorHandler.handleError)(response, error);
        }).catch(function () {
            return (0, _tractorErrorHandler.handleError)(response, new _tractorErrorHandler.TractorError('Could not save "' + itemPath + '"'));
        });
    };
}

// Errors:
// Dependencies: