'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getCopyPath = getCopyPath;
exports.getItemPath = getItemPath;
exports.respondOkay = respondOkay;
exports.respondItemNotFound = respondItemNotFound;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _Directory = require('../structure/Directory');

var _tractorErrorHandler = require('tractor-error-handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Constants:
var ITEM_NOT_FOUND_STATUS = 404;
var OKAY_STATUS = 200;

// Dependencies:


// Errors:
function getCopyPath(item) {
    var isDirectory = item instanceof _Directory.Directory;
    var collection = isDirectory ? item.directory.directories : item.directory.files;
    return _path2.default.join(item.directory.path, getUniqueName(collection, item));
}

function getItemPath(fileStructure, itemUrl) {
    itemUrl = itemUrl.replace(/\/$/, '').replace(/\//g, _path2.default.sep);

    var cleanUrl = decodeURIComponent(itemUrl);

    return _path2.default.resolve(process.cwd(), _path2.default.join(fileStructure.path, cleanUrl));
}

function respondOkay(response) {
    response.sendStatus(OKAY_STATUS);
}

function respondItemNotFound(itemPath, response) {
    (0, _tractorErrorHandler.handleError)(response, new _tractorErrorHandler.TractorError('Could not find "' + itemPath + '"', ITEM_NOT_FOUND_STATUS));
}

function getUniqueName(items, item) {
    var extension = item.extension || '';
    var n = 1;
    var uniqueName = item.name;
    while (nameExists(items, uniqueName)) {
        uniqueName = (item.basename + ' (' + n + ')' + extension).trim();
        n += 1;
    }
    return uniqueName;
}

function nameExists(items, name) {
    return !!items.find(function (item) {
        return item.name === name;
    });
}