'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Directory = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); // Dependencies:


// Errors:


var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _gracefulFs = require('graceful-fs');

var _gracefulFs2 = _interopRequireDefault(_gracefulFs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _File = require('./File');

var _tractorErrorHandler = require('tractor-error-handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Directory = exports.Directory = function () {
    function Directory(directoryPath, fileStructure) {
        _classCallCheck(this, Directory);

        this.path = _path2.default.resolve(process.cwd(), directoryPath);
        this.fileStructure = fileStructure;

        var isRoot = this.path === fileStructure.path;
        var isWithinRoot = this.path.indexOf('' + fileStructure.path + _path2.default.sep) === 0;

        if (!isRoot && !isWithinRoot) {
            throw new _tractorErrorHandler.TractorError('Cannot create "' + this.path + '" because it is outside of the root of the FileStructure');
        }

        this.init();

        var relativePath = _path2.default.relative(this.fileStructure.path, this.path);
        this.name = _path2.default.basename(this.path);
        this.basename = this.name;
        this.url = _path2.default.normalize('/' + relativePath).replace(/\\/, '/');

        if (isRoot) {
            this.parent = this.fileStructure;
        } else {
            var parentPath = _path2.default.dirname(this.path);
            var parent = fileStructure.allDirectoriesByPath[parentPath];
            if (parent) {
                this.directory = parent;
            } else {
                this.directory = new Directory(parentPath, fileStructure);
            }
            this.parent = this.directory;
        }
        this.parent.addItem(this);
    }

    _createClass(Directory, [{
        key: 'addItem',
        value: function addItem(item) {
            var itemIsDirectory = item instanceof Directory;
            var collection = itemIsDirectory ? this.directories : this.files;
            var allCollection = itemIsDirectory ? this.allDirectories : this.allFiles;

            if (item.directory === this && collection.indexOf(item) === -1) {
                collection.push(item);
            }
            if (allCollection.indexOf(item) === -1) {
                allCollection.push(item);
            }

            this.parent.addItem(item);
        }
    }, {
        key: 'cleanup',
        value: function cleanup() {
            var _this = this;

            return this.delete().then(function () {
                if (_this.directory) {
                    return _this.directory.cleanup();
                }
                return _bluebird2.default.resolve();
            }).catch(function (e) {
                if (e instanceof _tractorErrorHandler.TractorError) {
                    return;
                }
                throw e;
            });
        }
    }, {
        key: 'delete',
        value: function _delete() {
            var _this2 = this;

            if (!this.directories.length && !this.files.length) {
                return _gracefulFs2.default.rmdirAsync(this.path).then(function () {
                    _this2.parent.removeItem(_this2);
                });
            } else {
                return _bluebird2.default.reject(new _tractorErrorHandler.TractorError('Cannot delete "' + this.path + '" because it is not empty'));
            }
        }
    }, {
        key: 'init',
        value: function init() {
            this.directories = [];
            this.allDirectories = [];
            this.files = [];
            this.allFiles = [];
        }
    }, {
        key: 'move',
        value: function move(update) {
            var _this3 = this;

            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var isCopy = options.isCopy;

            update.oldPath = this.path;

            var newDirectory = new this.constructor(update.newPath, this.fileStructure);
            return newDirectory.save().then(function () {
                var items = _this3.directories.concat(_this3.files);
                return _bluebird2.default.map(items, function (item) {
                    var newPath = item.path.replace(update.oldPath, update.newPath);
                    return item.move({ newPath: newPath }, options);
                });
            }).then(function () {
                return isCopy ? null : _this3.delete();
            }).then(function () {
                return newDirectory;
            });
        }
    }, {
        key: 'read',
        value: function read() {
            var _this4 = this;

            if (this.reading) {
                return this.reading;
            }
            this.reading = _gracefulFs2.default.readdirAsync(this.path).then(function (itemPaths) {
                return readItems.call(_this4, itemPaths);
            });
            this.reading.then(function () {
                _this4.reading = null;
            });
            return this.reading;
        }
    }, {
        key: 'refresh',
        value: function refresh() {
            var _this5 = this;

            if (this.refreshing) {
                return this.refreshing;
            }
            this.init();
            this.refreshing = this.read();
            this.refreshing.then(function () {
                _this5.refreshing = null;
            });
            return this.refreshing;
        }
    }, {
        key: 'removeItem',
        value: function removeItem(item) {
            var itemIsDirectory = item instanceof Directory;
            var collection = itemIsDirectory ? this.directories : this.files;
            var allCollection = itemIsDirectory ? this.allDirectories : this.allFiles;

            var removeIndex = allCollection.indexOf(item);
            allCollection.splice(removeIndex, 1);
            if (item.directory === this) {
                var _removeIndex = collection.indexOf(item);
                collection.splice(_removeIndex, 1);
            }

            this.parent.removeItem(item);
        }
    }, {
        key: 'rimraf',
        value: function rimraf() {
            var _this6 = this;

            return _bluebird2.default.map(this.directories, function (directory) {
                return directory.rimraf();
            }).then(function () {
                return _bluebird2.default.map(_this6.files, function (file) {
                    return file.delete();
                });
            }).then(function () {
                return _gracefulFs2.default.rmdirAsync(_this6.path);
            }).then(function () {
                _this6.parent.removeItem(_this6);
            });
        }
    }, {
        key: 'save',
        value: function save() {
            var _this7 = this;

            return _gracefulFs2.default.statAsync(this.path).then(function () {
                return _bluebird2.default.resolve();
            }).catch(function () {
                if (_this7.directory) {
                    return _this7.directory.save().then(function () {
                        return _gracefulFs2.default.mkdirAsync(_this7.path);
                    });
                }
                return _bluebird2.default.resolve();
            });
        }
    }, {
        key: 'serialise',
        value: function serialise() {
            return this.toJSON();
        }
    }, {
        key: 'toJSON',
        value: function toJSON() {
            var basename = this.basename,
                directories = this.directories,
                files = this.files,
                path = this.path,
                url = this.url;

            directories = directories.sort(sortNames).map(function (directory) {
                return directory.serialise();
            });
            files = files.sort(sortNames).map(function (file) {
                return file.serialise();
            });
            return { basename: basename, directories: directories, files: files, isDirectory: true, path: path, url: url };
        }
    }]);

    return Directory;
}();

function getItemInfo(itemPath) {
    var _this8 = this;

    return _gracefulFs2.default.statAsync(itemPath).then(function (stat) {
        return handleItem.call(_this8, itemPath, stat);
    });
}

function handleItem(itemPath, stat) {
    if (stat.isDirectory()) {
        var directory = new this.constructor(itemPath, this.fileStructure);
        return directory.read();
    } else {
        var fileConstructor = this.fileStructure.getFileConstructor(itemPath);
        var file = new fileConstructor(itemPath, this.fileStructure);
        if (fileConstructor === _File.File) {
            return file;
        } else {
            return file.read();
        }
    }
}

function readItems(itemPaths) {
    var _this9 = this;

    return _bluebird2.default.map(itemPaths, function (itemPath) {
        return getItemInfo.call(_this9, _path2.default.join(_this9.path, itemPath));
    });
}

function sortNames(a, b) {
    var aName = a.name;
    var bName = b.name;
    if (aName === bName) {
        return 0;
    } else if (aName > bName) {
        return 1;
    } else {
        return -1;
    }
}