'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.File = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); // Dependencies:


// Errors:


var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _gracefulFs = require('graceful-fs');

var _gracefulFs2 = _interopRequireDefault(_gracefulFs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _Directory = require('./Directory');

var _tractorErrorHandler = require('tractor-error-handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var File = exports.File = function () {
    function File(filePath, fileStructure) {
        _classCallCheck(this, File);

        this.path = _path2.default.resolve(process.cwd(), filePath);
        this.fileStructure = fileStructure;

        var isWithinRoot = this.path.indexOf('' + fileStructure.path + _path2.default.sep) === 0;

        if (!isWithinRoot) {
            throw new _tractorErrorHandler.TractorError('Cannot create "' + this.path + '" because it is outside of the root of the FileStructure');
        }

        this.url = ('/' + _path2.default.relative(this.fileStructure.path, this.path)).replace(/\\/g, '/');
        this.name = _path2.default.basename(this.path);
        this.extension = this.extension || _path2.default.extname(this.path);
        this.basename = _path2.default.basename(this.path, this.extension);

        var parentPath = _path2.default.dirname(this.path);
        var parent = fileStructure.allDirectoriesByPath[parentPath];
        if (parent) {
            this.directory = parent;
        } else {
            this.directory = new _Directory.Directory(parentPath, fileStructure);
        }
        this.directory.addItem(this);
    }

    _createClass(File, [{
        key: 'addReference',
        value: function addReference(reference) {
            this.fileStructure.referenceManager.addReference(this, reference);
        }
    }, {
        key: 'cleanup',
        value: function cleanup() {
            var _this = this;

            return this.delete().then(function () {
                return _this.directory.cleanup();
            });
        }
    }, {
        key: 'clearReferences',
        value: function clearReferences() {
            this.fileStructure.referenceManager.clearReferences(this.path);
            this.fileStructure.referenceManager.clearReferencedBy(this.path);
        }
    }, {
        key: 'delete',
        value: function _delete() {
            var _this2 = this;

            var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
            var isMove = options.isMove;


            if (!isMove && this.referencedBy.length) {
                return _bluebird2.default.reject(new _tractorErrorHandler.TractorError('Cannot delete ' + this.path + ' as it is referenced by another file.'));
            }

            return _gracefulFs2.default.unlinkAsync(this.path).then(function () {
                _this2.directory.removeItem(_this2);
                _this2.fileStructure.referenceManager.clearReferences(_this2.path);
            });
        }
    }, {
        key: 'move',
        value: function move(update) {
            var _this3 = this;

            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var isCopy = options.isCopy;

            update.oldPath = this.path;

            var newFile = new this.constructor(update.newPath, this.fileStructure);

            options.isMove = true;
            var save = newFile.save(this.buffer, options);

            if (isCopy) {
                return save;
            }

            var referencedBy = this.referencedBy;

            this.clearReferences();

            var nameChange = {
                oldName: this.basename,
                newName: newFile.basename
            };
            return save.then(function () {
                return _this3.delete(options);
            }).then(function () {
                return newFile.refactor('fileNameChange', nameChange);
            }).then(function () {
                return _bluebird2.default.map(referencedBy, function (reference) {
                    var extension = reference.extension;

                    var referenceNameChange = _extends({}, nameChange, {
                        extension: extension
                    });
                    var referencePathChange = {
                        fromPath: reference.path,
                        oldToPath: _this3.path,
                        newToPath: newFile.path
                    };
                    reference.addReference(newFile);
                    return reference.refactor('referenceNameChange', referenceNameChange).then(function () {
                        return reference.refactor('referencePathChange', referencePathChange);
                    });
                }).catch(function () {
                    throw new _tractorErrorHandler.TractorError('Could not update references after moving ' + _this3.path + '.');
                });
            });
        }
    }, {
        key: 'read',
        value: function read() {
            var _this4 = this;

            return _gracefulFs2.default.readFileAsync(this.path).then(function (buffer) {
                return setData.call(_this4, buffer);
            }).then(function () {
                return _this4.content;
            });
        }
    }, {
        key: 'refactor',
        value: function refactor() {
            return _bluebird2.default.resolve();
        }
    }, {
        key: 'save',
        value: function save(data) {
            var _this5 = this;

            return this.directory.save().then(function () {
                return _gracefulFs2.default.writeFileAsync(_this5.path, data);
            }).then(function () {
                return setData.call(_this5, new Buffer(data));
            }).then(function () {
                return _this5.content;
            });
        }
    }, {
        key: 'serialise',
        value: function serialise() {
            return this.toJSON();
        }
    }, {
        key: 'toJSON',
        value: function toJSON() {
            var references = this.references.map(getFileDetails);
            var referencedBy = this.referencedBy.map(getFileDetails);
            var details = getFileDetails(this);
            return _extends({ references: references, referencedBy: referencedBy }, details);
        }
    }, {
        key: 'references',
        get: function get() {
            return this.fileStructure.referenceManager.getReferences(this.path);
        }
    }, {
        key: 'referencedBy',
        get: function get() {
            return this.fileStructure.referenceManager.getReferencedBy(this.path);
        }
    }]);

    return File;
}();

function getFileDetails(file) {
    var basename = file.basename,
        extension = file.extension,
        path = file.path,
        url = file.url;

    return { basename: basename, extension: extension, path: path, url: url };
}

function setData(data) {
    this.buffer = data;
    this.content = this.buffer.toString();
    this.data = this.content;
}