'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.FileStructure = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _events = require('events');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _tractorLogger = require('tractor-logger');

var _Directory = require('./Directory');

var _File = require('./File');

var _ReferenceManager = require('./ReferenceManager');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Constants:
var DOT_FILE_REGEX = /(^|[/\\])\../;
var EXTENSION_MATCH_REGEX = /[^.]*(\..*)?/;

// Dependencies:

var FileStructure = exports.FileStructure = function () {
    function FileStructure(fsPath) {
        _classCallCheck(this, FileStructure);

        this.fileTypes = {};
        this.path = _path2.default.resolve(process.cwd(), fsPath);

        this.init();
    }

    _createClass(FileStructure, [{
        key: 'addFileType',
        value: function addFileType(fileConstructor) {
            this.fileTypes[fileConstructor.prototype.extension] = fileConstructor;
        }
    }, {
        key: 'addItem',
        value: function addItem(item) {
            var collection = item instanceof _Directory.Directory ? this.allDirectoriesByPath : this.allFilesByPath;
            collection[item.path] = item;
        }
    }, {
        key: 'getFileConstructor',
        value: function getFileConstructor(filePath) {
            var fileName = _path2.default.basename(filePath);

            var _fileName$match = fileName.match(EXTENSION_MATCH_REGEX),
                _fileName$match2 = _slicedToArray(_fileName$match, 2),
                fullExtension = _fileName$match2[1];

            var extension = _path2.default.extname(fileName);
            return this.fileTypes[fullExtension] || this.fileTypes[extension] || _File.File;
        }
    }, {
        key: 'init',
        value: function init() {
            this.allFilesByPath = {};
            this.allDirectoriesByPath = {};
            this.structure = new _Directory.Directory(this.path, this);
            this.referenceManager = new _ReferenceManager.ReferenceManager(this);
        }
    }, {
        key: 'read',
        value: function read() {
            return this.structure.read();
        }
    }, {
        key: 'removeItem',
        value: function removeItem(item) {
            var collection = item instanceof _Directory.Directory ? this.allDirectoriesByPath : this.allFilesByPath;
            collection[item.path] = null;
        }
    }, {
        key: 'watch',
        value: function watch() {
            var _this = this;

            (0, _tractorLogger.info)('Watching ' + this.path + ' for changes...');
            var watcher = new _events.EventEmitter();
            _chokidar2.default.watch(this.path, {
                ignored: DOT_FILE_REGEX,
                ignoreInitial: true,
                awaitWriteFinish: true
            }).on('all', function (event, itemPath) {
                var changeDirectory = void 0;
                if (itemPath === _this.path) {
                    changeDirectory = _this.structure;
                } else {
                    changeDirectory = _this.allDirectoriesByPath[_path2.default.dirname(itemPath)];
                }
                changeDirectory.refresh().then(function () {
                    watcher.emit('change');
                });
            });
            return watcher;
        }
    }]);

    return FileStructure;
}();