'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.copyFile = copyFile;
exports.createDir = createDir;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _gracefulFs = require('graceful-fs');

var _gracefulFs2 = _interopRequireDefault(_gracefulFs);

var _tractorLogger = require('tractor-logger');

var _tractorErrorHandler = require('tractor-error-handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Constants:
var ALREADY_EXISTS = 'EEXIST';
var DOESNT_EXIST = 'ENOENT';

// Dependencies:


// Errors:
function copyFile(readPath, writePath) {
    return _gracefulFs2.default.readFileAsync(writePath).then(function () {
        return throwExists(writePath);
    }).catch(_bluebird2.default.OperationalError, function (error) {
        if (error && error.cause && error.cause.code === DOESNT_EXIST) {
            logCreating(writePath);
        } else {
            throw error;
        }
    }).then(function () {
        return _gracefulFs2.default.readFileAsync(readPath);
    }).then(function (contents) {
        return _gracefulFs2.default.writeFileAsync(writePath, contents);
    }).then(function () {
        return logCreated(writePath);
    });
}

function createDir(directoryPath) {
    logCreating(directoryPath);
    return _gracefulFs2.default.mkdirAsync(directoryPath).then(function () {
        return logCreated(directoryPath);
    }).catch(_bluebird2.default.OperationalError, function (error) {
        if (error && error.cause && error.cause.code === ALREADY_EXISTS) {
            throwExists(directoryPath);
        } else {
            throw error;
        }
    });
}

function logCreating(path) {
    (0, _tractorLogger.info)('Creating "' + path + '"...');
}

function logCreated(path) {
    (0, _tractorLogger.info)('"' + path + '" created.');
}

function throwExists(path) {
    throw new _tractorErrorHandler.TractorError('"' + path + '" already exists.');
}